%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%* MATLAB code to run simulations of Prizak, Ezard & Hoyle 2014, Ecol. Evol.
clear
close
cd('~/Dropbox/transgen/grandparents')

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%* parameter definitions
tau=0.25;		% fraction of a generation between critical period of development and selection 
rho=0.25;		% environmental auto-correlation
xi_bar=0;		% mean of stationary autocorrelated noise
sig_xi=2;		% standard deviation of stationary autocorrelated noise
omega2=50;		% width of fitness function, 3=> strong stabilizing selection
omegaB2=90;		% width of fitness function, 3=> strong stabilizing selection
sigma_e2=.5;	% variance in residual component of phenotypic variation
A=0;			% intercept (elevation) of evolution in reference environment
B=2;			% slope of reaction norm in reference environment
Wmax=1;			% maximal fitness
%m=0.5;			% m!=0  => maternal effects
%g=0;			% g!=0  => grandmaternal effects
noPlas=false;	% holds bbar at 0 for the special case included in fig 2 of Lande 2009
expec=false;	% forces xi the to be 0 for all environments
Gaa=0.5;		% variance of elevation, hold %GaaGbbrat = (Gbb*delta^2)/(Gaa+Gbb*delta^2) near 1 in Lande
Gab=0;			% assume zero covariance 
Gba=0; 			% assume zero covariance 
Gbb=0.045;		% variance of plasticity
delta=10;		% size of step change
nrp=1e5;		% number of generations
Ut=10;

%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
%*%*%* run over a all combinations of g and m values, 
mm=-0.25:.5:0.25;
lm=length(mm);
gg=-0.15:0.3:0.15;
lg=length(gg);
 
%*%*%* somewhere to put stuff
fot=zeros(lm, lg);
cot=zeros(lm, lg);

for j=1:lg;
  g=gg(j)
  for i=1:lm;
     m=mm(i)
 
     %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
     %*%* define things to receive variables and initiate
     %*%* initiate at the equilibrium solutions in reference environment, i.e delta=0
     eabar=zeros(nrp,1);
     eabar(1:3)=(1-m-g)*A;
     ebbar=zeros(nrp,1);
     ebbar(1:3)=rho*B/(1+m^2);
     ezbar=zeros(nrp,1);
     ezsbr=zeros(nrp,1);
     ezsbr(1:3)=A;
     ezbar(1:3)=A;
     efitness=zeros(nrp,1);
     efitness(1:3)=1;

     %*%* some other short hands
     vxi=sig_xi^2;
     GG=[Gaa Gab; Gba Gbb];

     for t=3:nrp
       %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
       %*%*%*%*%*%*%*%* obtain environments during development eps_tT and at selection eps_t
       env_ed = ((t-1-tau)>=Ut)*delta;
       env_devel = ((t-tau)>=Ut)*delta;
       env_selct = (t>=Ut)*delta;

       %*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*%*
       %*%*%*%*%*%*%*%* calculate phenotypic variance at time t from ....
       denom = 1 - m^2 - g^2 - ((2*m^2*g)/(1-g));
       p1 = Gaa + Gbb*env_devel*env_ed;
       esigz2t = (Gaa +Gbb*(env_devel^2) + sigma_e2 + Gbb*vxi + (((4*m)/(4-2*m-g))*p1) + (((2*g)/(4-2*m-g))*p1) + (((4*m*g)/((1-g)*(4-2*m-g)))*p1))/denom;
       egammaStar = 1/(omega2 + esigz2t);

       %*%*%*%*%*%*%*%* some book-keeping to help calculate change in elevation & plasticity 
       ezbar(t) = eabar(t) + ebbar(t)*env_devel + m*ezsbr(t-1) + g*ezsbr(t-2);
       etheta = A + B*env_selct;
       eDisc= ezbar(t)-etheta;
                  
       edela = -egammaStar*Gaa*(eDisc*(1+m));
       edelb = -egammaStar*Gbb*(eDisc*(env_devel + m*env_ed)) - egammaStar*Gbb*vxi*(ebbar(t)*(1+m^2)-rho*B);

       %*%*%*%*%*%*%*%* calculate expected fitness 
       efitness(t) = sqrt(egammaStar*omega2)*exp(-(egammaStar/2) * eDisc^2);
         
       %*%*%*%*%*%*%*%* update elevation & plasticity
       eabar(t+1) = eabar(t) + edela;
       ebbar(t+1) = ebbar(t) + edelb;

       %*%*%*%*%*%*%*%* update zstarbar to link generation n to generation n-1 
       p1 = ezbar(t) - egammaStar*Gbb*env_devel*vxi*((ebbar(t)*(1+m^2))-rho*B);
       p2 = egammaStar*Gbb*(env_devel + m*env_ed)*(ebbar(t)-B*rho)*vxi;
       p3 = egammaStar*(Gaa*(1+m) + Gbb*(env_devel*(env_devel+m*env_ed) + vxi));
       p4 = eDisc;         
       ezsbr(t) = p1- p2 - (p3*p4);
     end
        
   %*%*%*%*%* output the equilibrium fitness level
   efitness(1:2) = efitness(3);	%%assign first two values to third value (at initial equilibrium) for ease of plotting
   fid = fopen([date '_mg_del10_xi' num2str(sig_xi) '_m' num2str(m) '_g' num2str(g) '_efot.txt'], 'w');
   fprintf(fid, '%12.8f\n', efitness);
   fclose(fid);      
   end;
 end;

